using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using PIbd_24_EredavkinRA_BusBusiness.Repositories;
using PIbd_24_EredavkinRA_BusBusiness.Repositories.Implementations;
using Serilog;
using Unity;
using Unity.Lifetime;
using Unity.Microsoft.Logging;

namespace PIbd_24_EredavkinRA_BusBusiness
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormBus>());
        } 

        private static IUnityContainer CreateContainer()
        {
            var container = new UnityContainer();
            container.AddExtension(new LoggingExtension(CreateLoggerFactory()));
            container.RegisterType<IBusRepository, BusRepository>(new TransientLifetimeManager());
            container.RegisterType<IRoutRepositories, RoutRepositories>(new TransientLifetimeManager());
            container.RegisterType<IRoutSheetRepository, RoutSheetRepository>(new TransientLifetimeManager());
            container.RegisterType<IStaffRepositories, StaffRepository>(new TransientLifetimeManager());
            container.RegisterType<IToRepository, ToRepository>(new TransientLifetimeManager());

            container.RegisterType<IConnectionString, ConnectionString>(new SingletonLifetimeManager());
            return container;
        }
        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                    .SetBasePath(Directory.GetCurrentDirectory())
                    .AddJsonFile("appsettings.json")
                    .Build())
                .CreateLogger());
            return loggerFactory;
        }
    }
}